# Generated by Django 3.2.16 on 2022-12-23 12:54

from django.db import migrations


def migrate_group_to_meta(apps, schema_editor):
    Template = apps.get_model("api", "Template")

    for template in Template.objects.filter(group__isnull=False):
        template.meta["group"] = template.group
        template.save()


def migrate_group_to_meta_reverse(apps, schema_editor):
    Template = apps.get_model("api", "Template")

    for template in Template.objects.filter(meta__has_key="group"):
        template.group = template.meta["group"]
        del template.meta["group"]
        template.save()


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0005_xlsx_template_engine"),
    ]

    operations = [
        migrations.RunPython(migrate_group_to_meta, migrate_group_to_meta_reverse),
        migrations.RemoveField(
            model_name="template",
            name="group",
        ),
    ]



from django.urls import re_path
from rest_framework.routers import DefaultRouter

from . import views

r = DefaultRouter()

r.register("template", views.TemplateView)

urlpatterns = [
    re_path(
        r"^template-download/(?P<pk>.+)$",
        views.DownloadTemplateView.as_view(),
        name="template-download",
    ),
    re_path(
        r"^convert$",
        views.ConvertView.as_view(),
        name="convert",
    ),
]

urlpatterns.extend(r.urls)



import os

import pytest
from django.core.management import call_command

from document_merge_service.api.data import django_file


@pytest.mark.parametrize("dry", [True, False])
def test_clean_dangling_files(db, dry, settings, template_factory):
    templates = [
        template_factory(template=django_file("docx-template.docx")),
        template_factory(template=django_file("docx-template-syntax.docx")),
    ]
    dangling_files = [
        django_file("docx-template-filters.docx"),
        django_file("docx-template-loopcontrols.docx"),
    ]

    call_command("clean_dangling_files", dry=dry)

    assert (
        all(
            [
                os.path.isfile(os.path.join(settings.MEDIA_ROOT, file.name)) is dry
                for file in dangling_files
            ]
        )
        is True
    )
    assert (
        all([os.path.isfile(template.template.path) is True for template in templates])
        is True
    )



import sentry_sdk


def test_sentry(sentry_mock):
    assert len(sentry_mock.method_calls) == 0
    sentry_sdk.capture_exception(Exception("test_sentry_exc"))
    assert len(sentry_mock.method_calls) == 1
    sentry_mock.record_lost_event.assert_not_called()
    assert (
        sentry_mock.method_calls[0]
        .args[0]
        .items[0]
        .get_event()["exception"]["values"][0]["value"]
        == "test_sentry_exc"
    )



from django.core.files.storage import DefaultStorage
from django.db import models
from django.dispatch import receiver


class Template(models.Model):
    DOCX_TEMPLATE = "docx-template"
    DOCX_MAILMERGE = "docx-mailmerge"
    XLSX_TEMPLATE = "xlsx-template"
    ENGINE_CHOICES_LIST = (DOCX_TEMPLATE, DOCX_MAILMERGE, XLSX_TEMPLATE)
    ENGINE_CHOICES_TUPLE = (
        (
            DOCX_TEMPLATE,
            "Docx Template engine (https://github.com/elapouya/python-docx-template)",
        ),
        (
            DOCX_MAILMERGE,
            "Docx MailMerge engine (https://github.com/Bouke/docx-mailmerge)",
        ),
        (
            XLSX_TEMPLATE,
            "Xlsx Template engine (https://github.com/zhangyu836/python-xlsx-template)",
        ),
    )

    slug: models.SlugField = models.SlugField(primary_key=True)
    description: models.TextField = models.TextField(default="")
    template: models.FileField = models.FileField(max_length=1024)
    engine: models.CharField = models.CharField(
        max_length=20, choices=ENGINE_CHOICES_TUPLE
    )
    meta = models.JSONField(default=dict)


@receiver(models.signals.post_delete, sender=Template)
def auto_delete_file_on_delete(sender, instance, **kwargs):
    """Delete template file from filesystem when `Template` object is deleted."""

    DefaultStorage().delete(instance.template.name)


@receiver(models.signals.pre_save, sender=Template)
def auto_delete_file_on_change(sender, instance, **kwargs):
    """Delete old template file from filesystem when `Template` is given a new template file."""

    try:
        old_file = Template.objects.get(pk=instance.pk).template
    except Template.DoesNotExist:
        return

    if old_file:
        new_file = instance.template
        if not old_file == new_file:
            DefaultStorage().delete(old_file.name)



import hashlib
import os
import shutil

import pytest
from django.core.management import call_command
from faker import Faker

from document_merge_service.api.data import django_file


def sha256sum(path):
    with open(path, "rb") as file:
        return hashlib.sha256(file.read()).hexdigest()


def is_equal(a, b):
    return sha256sum(a) == sha256sum(b)


@pytest.fixture
def tmp_path(settings):
    path = os.path.join(settings.MEDIA_ROOT, f"local-templates-{Faker().uuid4()}")

    yield path

    shutil.rmtree(path, ignore_errors=True)


@pytest.mark.parametrize("dry", [True, False])
def test_upload_local_templates(db, dry, template_factory, tmp_path):
    templates = [
        template_factory(template=django_file("docx-template-syntax.docx")),
        template_factory(template=django_file("docx-template-syntax.docx")),
    ]

    files = [
        django_file(
            "docx-template.docx",
            new_path=tmp_path,
            new_name=templates[0].template.name,
        ),
        django_file(
            "docx-template.docx",
            new_path=tmp_path,
            new_name=templates[1].template.name,
        ),
        django_file(
            "docx-template.docx",
            new_path=tmp_path,
            new_name="some-file-without-template.docx",
        ),
    ]

    paths = [os.path.join(tmp_path, file.name) for file in files]

    assert not is_equal(templates[0].template.path, paths[0])
    assert not is_equal(templates[1].template.path, paths[1])

    call_command("upload_local_templates", dry=dry, source=f"{tmp_path}/*.docx")

    assert is_equal(templates[0].template.path, paths[0]) != dry
    assert is_equal(templates[1].template.path, paths[1]) != dry



# Generated by Django 3.2.9 on 2021-11-01 12:34

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0002_template_group"),
    ]

    operations = [
        migrations.AddField(
            model_name="template",
            name="meta",
            field=models.JSONField(default=dict),
        ),
    ]



from functools import singledispatch

from django.conf import settings
from django.urls import reverse
from generic_permissions.validation import ValidatorMixin
from rest_framework import exceptions, serializers

from . import engines, models


class CustomFileField(serializers.FileField):
    """
    Custom FileField.

    `to_representation()` of this FileField returns the file object instead of just the
    filename.
    """

    def to_representation(self, value):
        return value or None


class TemplateFileField(serializers.FileField):
    def get_attribute(self, instance):
        # Hacky workaround - we need the instance in `to_representation()`,
        # not the field value.
        # We cannot use `parent.instance`, as that won't be set to
        # the current instance in a list view
        return instance

    def to_representation(self, value):
        if value and value.pk and value.template:
            return reverse("template-download", args=[value.pk])


class TemplateSerializer(ValidatorMixin, serializers.ModelSerializer):
    disable_template_validation = serializers.BooleanField(
        allow_null=True, default=False
    )
    available_placeholders = serializers.ListField(allow_null=True, required=False)
    sample_data = serializers.JSONField(allow_null=True, required=False)
    files = serializers.ListField(
        child=CustomFileField(write_only=True, allow_empty_file=False), required=False
    )
    template = TemplateFileField()

    def _sample_to_placeholders(self, sample_doc):
        @singledispatch
        def _doc(doc):
            return ""

        @_doc.register(dict)
        def _(doc):
            return [f"{k}.{name}" for k, v in doc.items() for name in _doc(v)] + [
                k for k in doc.keys()
            ]

        @_doc.register(list)
        def _(doc):
            res = []
            for item in doc:
                res.extend([f"[].{var}" if var else "[]" for var in _doc(item)])
                res.append("[]")
            return list(set(res))

        return sorted([x.replace(".[]", "[]") for x in _doc(sample_doc)])

    def validate(self, data):
        if data.pop("disable_template_validation", False):
            # Some template structures cannot be validated automatically,
            # or it would be impossible or too much effort to provide accurate
            # sample data. For those cases, we allow disabling the validation.
            return data

        engine = data.get("engine", self.instance and self.instance.engine)
        template = data.get("template", self.instance and self.instance.template)

        available_placeholders = data.pop("available_placeholders", None)
        sample_data = data.pop("sample_data", None)
        files = data.pop("files", None)

        if sample_data and available_placeholders:
            raise exceptions.ValidationError(
                "Only one of available_placeholders and sample_data is allowed"
            )
        elif files and engine != models.Template.DOCX_TEMPLATE:
            raise exceptions.ValidationError(
                f'Files are only accepted with the "{models.Template.DOCX_TEMPLATE}"'
                f" engine"
            )
        elif sample_data:
            if files:
                for file in files:
                    sample_data[file.name] = file

            available_placeholders = self._sample_to_placeholders(sample_data)
        elif files:
            raise exceptions.ValidationError(
                "Files are only accepted when also providing sample_data"
            )

        engine = engines.get_engine(engine, template)
        engine.validate(
            available_placeholders=available_placeholders, sample_data=sample_data
        )

        return data

    class Meta:
        model = models.Template
        fields = (
            "slug",
            "description",
            "template",
            "engine",
            "available_placeholders",
            "sample_data",
            "files",
            "disable_template_validation",
            "meta",
        )


class TemplateMergeSerializer(serializers.Serializer):
    data = serializers.JSONField(
        required=True, help_text="Data as json used for merging"
    )
    convert = serializers.ChoiceField(
        allow_null=True,
        required=False,
        choices=settings.UNOCONV_ALLOWED_TYPES,
        help_text="Optionally convert result document to this type.",
    )
    files = serializers.ListField(
        child=CustomFileField(write_only=True, allow_empty_file=False), required=False
    )

    class Meta:
        model = models.Template


class ConvertSerializer(serializers.Serializer):
    file = CustomFileField(required=True, allow_empty_file=False)
    target_format = serializers.ChoiceField(
        allow_null=False,
        required=True,
        choices=[("pdf", "PDF")],
        help_text="The target format of the conversion. Currently only 'pdf' is supported.",
    )



import glob
import os

from django.core.files.storage import DefaultStorage
from django.core.management.base import BaseCommand

from document_merge_service.api.models import Template


class Command(BaseCommand):
    help = "Upload local template files to configured storage backend"

    def add_arguments(self, parser):
        parser.add_argument(
            "-s",
            "--source",
            help="Glob-style path to the template files that should be uploaded. E.g. `/tmp/templates/*.docx`",
            dest="source",
            type=str,
            required=True,
        )
        parser.add_argument(
            "--dry-run",
            help="Only show what files would be uploaded to the storage backend; don't actually upload them.",
            dest="dry",
            action="store_true",
            default=False,
        )

    def handle(self, *args, **options):
        storage = DefaultStorage()

        for path in glob.iglob(options["source"]):
            filename = os.path.basename(path)

            try:
                template = Template.objects.get(template=filename)
            except Template.DoesNotExist:
                self.stdout.write(
                    self.style.WARNING(f'No template for filename "{filename}" found')
                )
                continue

            if not options.get("dry"):
                try:
                    with open(path, "rb") as file:
                        storage.delete(template.template.name)
                        storage.save(template.template.name, file)

                    self.stdout.write(
                        self.style.SUCCESS(
                            f'Uploaded file for template "{template.pk}"'
                        )
                    )
                except Exception as e:  # pragma: no cover
                    self.stdout.write(
                        self.style.ERROR(
                            f'Could not upload file for template "{template.pk}": {str(e)}'
                        )
                    )
            else:
                self.stdout.write(
                    self.style.WARNING(
                        f'Would upload file for template "{template.pk}"'
                    )
                )



import os

from django.conf import settings
from django.core.management.base import BaseCommand

from document_merge_service.api.models import Template


class Command(BaseCommand):
    help = "Remove dangling template files that are not attached to a template model anymore. Currently only usable with local filesystem."

    def add_arguments(self, parser):
        parser.add_argument("--dry-run", dest="dry", action="store_true", default=False)

    def handle(self, *args, **options):
        used_files = [template.template.path for template in Template.objects.all()]

        for subdir, dirs, files in os.walk(settings.MEDIA_ROOT):
            for f in files:
                path = os.path.join(subdir, f)
                if path not in used_files and os.path.isfile(path):
                    try:
                        if not options.get("dry"):
                            os.remove(path)
                            self.stdout.write(
                                self.style.SUCCESS(f"Deleted dangling file '{path}'")
                            )
                        else:
                            self.stdout.write(
                                self.style.WARNING(
                                    f"Would delete dangling file '{path}'"
                                )
                            )
                    except Exception as e:  # pragma: no cover
                        self.stdout.write(
                            self.style.ERROR(
                                f"Could not delete dangling file '{path}': {str(e)}"
                            )
                        )



import os
import random
import shutil
import sys
from multiprocessing.pool import ThreadPool
from pathlib import Path
from subprocess import TimeoutExpired, run
from time import sleep

import pytest
from psutil import process_iter

from .. import unoconv
from ..unoconv import Unoconv, run_fork_safe


def test_timeout():
    with pytest.raises(TimeoutExpired):
        run_fork_safe(["sleep", "infinity"], timeout=0.5)


def kill_zombies():  # pragma: no cover
    # Depending on if we are pid 1, we want to cleanup zombie processes
    # As our pid depends on how the tests are run, we add "no cover" to this function
    if os.getpid() != 1:
        return
    for x in process_iter(["name"]):
        if x.status() == "zombie":
            x.wait()


def kill_dms_sleep(dms_test_bin):  # pragma: no cover
    found = False
    kill_zombies()
    for x in process_iter(["name"]):
        if dms_test_bin.name == x.name():
            found = True
            x.kill()
            x.wait()
    return found


def test_fork(dms_test_bin):  # pragma: no cover
    kill_dms_sleep(dms_test_bin)
    shell_cmd = f"{dms_test_bin} infinity & disown"
    run(["/bin/bash", "-c", shell_cmd])
    sleep(0.5)
    assert kill_dms_sleep(dms_test_bin)
    run_fork_safe(["/bin/bash", "-c", shell_cmd])
    sleep(0.5)
    assert not kill_dms_sleep(dms_test_bin)


def run_fork_load(test_file):
    conv = Unoconv("/usr/bin/python3", shutil.which("unoconv"))
    return conv.process(test_file, "pdf")


def test_unoconv_unshare_error(loadtest_data, caplog):
    test_file = Path(loadtest_data, "1.docx")
    conv = Unoconv("/usr/bin/python3", shutil.which("unoconv"))
    try:
        save = unoconv._unshare
        unoconv._unshare = "false"
        conv.process(test_file, "pdf")
        assert "CAP_SYS_ADMIN" in caplog.text
    finally:
        unoconv._unshare = save


def test_unoconv_error(caplog):
    test_file = "/asdfasdfa"
    conv = Unoconv("/usr/bin/python3", shutil.which("unoconv"))
    conv.process(test_file, "pdf")
    assert "unoconv failed with returncode" in caplog.text


def try_fork_load(arg):
    n, test_file = arg
    if n < 10:
        # slowly start load test
        sleep(0.05 * (10 - n))
    try:
        result = run_fork_load(test_file)
        return result
    except Exception as e:  # pragma: no cover
        return e


def test_fork_load(capsys, loadtest_data):
    count = 8
    test_files = []
    test_files += [Path(loadtest_data.parent, "docx-mailmerge.docx")] * count
    test_files += [Path(loadtest_data, "1.doc")] * count
    test_files += [Path(loadtest_data, "2.docx")] * count
    test_files += [Path(loadtest_data, "3.docx")] * count
    test_files += [Path(loadtest_data, "4.docx")] * count
    random.shuffle(test_files)
    try:
        pool = ThreadPool(8)
        with capsys.disabled():
            sys.stdout.write(" Loadtest: ")
            sys.stdout.flush()
        for result in pool.imap(try_fork_load, enumerate(test_files)):
            with capsys.disabled():
                sys.stdout.write(".")
                sys.stdout.flush()
            if isinstance(result, Exception):  # pragma: no cover
                raise result
            elif not result.stdout.startswith(b"%PDF"):  # pragma: no cover
                raise ValueError(result)
        with capsys.disabled():
            sys.stdout.write("done")
    finally:
        pool.close()
        pool.join()



import pytest
from django.urls import reverse
from rest_framework import status

from document_merge_service.api.data import django_file


@pytest.mark.parametrize(
    "filename,target_filename,target_format,response_content_type",
    [
        (
            "docx-template.docx",
            "docx-template.pdf",
            "pdf",
            "application/pdf",
        ),
        (
            "2023.test.test.docx-template.docx",
            "2023.test.test.docx-template.pdf",
            "pdf",
            "application/pdf",
        ),
    ],
)
def test_convert(
    db, client, filename, target_filename, target_format, response_content_type
):
    url = reverse("convert")
    file_to_convert = django_file(filename)

    data = {"file": file_to_convert.file, "target_format": target_format}
    response = client.post(url, data=data, format="multipart")

    assert response.status_code == status.HTTP_200_OK
    assert response.headers.get("Content-Type") == response_content_type
    assert (
        response.headers.get("Content-Disposition")
        == f'attachment; filename="{target_filename}"'
    )


def test_incorrect_file_type(db, client):
    url = reverse("convert")
    file_to_convert = django_file("invalid-template.xlsx")

    data = {"file": file_to_convert.file, "target_format": "pdf"}
    response = client.post(url, data=data, format="multipart")

    assert response.status_code == status.HTTP_400_BAD_REQUEST



import logging

import sentry_sdk
from sentry_sdk.integrations.django import DjangoIntegration
from sentry_sdk.integrations.logging import LoggingIntegration


def sentry_init(dsn, env, traces_sample_rate, send_default_pii):
    sentry_sdk.init(
        dsn=dsn,
        environment=env,
        send_default_pii=send_default_pii,
        traces_sample_rate=traces_sample_rate,
        integrations=[
            DjangoIntegration(),
            LoggingIntegration(level=logging.INFO, event_level=logging.ERROR),
            # the `level` kwarg defaults to INFO and instructs sentry to include log messages of that level or higher in
            # the message sent to sentry when triggered by an event of level specified in event_level kwarg as
            # breadcrumbs.
        ],
    )



import mimetypes
from os.path import splitext

import jinja2
from django.http import HttpResponse
from django.utils.encoding import smart_str
from generic_permissions.permissions import PermissionViewMixin
from generic_permissions.visibilities import VisibilityViewMixin
from rest_framework import exceptions, viewsets
from rest_framework.decorators import action
from rest_framework.generics import RetrieveAPIView
from rest_framework.views import APIView

from . import engines, filters, models, serializers
from .file_converter import FileConverter


class TemplateView(VisibilityViewMixin, PermissionViewMixin, viewsets.ModelViewSet):
    queryset = models.Template.objects
    serializer_class = serializers.TemplateSerializer
    filterset_class = filters.TemplateFilterSet
    ordering_fields = ("slug", "description")
    ordering = ("slug",)

    @action(
        methods=["post"],
        detail=True,
        serializer_class=serializers.TemplateMergeSerializer,
    )
    def merge(self, request, pk=None):
        template = self.get_object()
        engine = engines.get_engine(template.engine, template.template)

        content_type, _ = mimetypes.guess_type(template.template.name)
        response = HttpResponse(
            content_type=content_type or "application/force-download"
        )

        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        data = serializer.data["data"]
        files = serializer.data.get("files")

        if files is not None:
            for file in files:
                data[file.name] = file

        try:
            response = engine.merge(serializer.data["data"], response)
        except jinja2.UndefinedError as exc:
            raise exceptions.ValidationError(
                f"Placeholder from template not found in data: {exc}"
            )

        convert = serializer.data.get("convert")

        if convert:
            response = FileConverter.convert(response.content, convert)

        filename = f"{template.slug}.{convert}"
        response["Content-Disposition"] = f'attachment; filename="{filename}"'
        return response


class DownloadTemplateView(RetrieveAPIView):
    queryset = models.Template.objects
    lookup_field = "pk"

    def retrieve(self, request, **kwargs):
        template = self.get_object()

        mime_type, _ = mimetypes.guess_type(template.template.name)
        extension = mimetypes.guess_extension(mime_type)
        content_type = mime_type or "application/force-download"

        response = HttpResponse(content_type=content_type)
        response["Content-Disposition"] = 'attachment; filename="%s"' % smart_str(
            template.slug + extension
        )
        response["Content-Length"] = template.template.size
        response.write(template.template.read())
        return response


class ConvertView(APIView):
    def post(self, request, **kwargs):
        serializer = serializers.ConvertSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        file = serializer.data["file"]
        target_format = serializer.data["target_format"]

        content_type, foo = mimetypes.guess_type(file.name)

        if content_type not in [
            "application/vnd.oasis.opendocument.text",
            "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
        ]:
            raise exceptions.ValidationError(
                "Incorrect file format. Only docx and odt files are supported for conversion."
            )

        response = FileConverter.convert(file.read(), target_format)

        filename = f"{splitext(file.name)[0]}.{target_format}"
        response["Content-Disposition"] = f'attachment; filename="{filename}"'
        return response



import os
import re

import environ

from .sentry import sentry_init

env = environ.Env()
django_root = environ.Path(__file__) - 2

ENV_FILE = env.str("ENV_FILE", default=django_root(".env"))
if os.path.exists(ENV_FILE):  # pragma: no cover
    environ.Env.read_env(ENV_FILE)

# per default production is enabled for security reasons
# for development create .env file with ENV=development
ENV = env.str("ENV", "production")


def default(default_dev=env.NOTSET, default_prod=env.NOTSET):
    """Environment aware default."""
    return default_prod if ENV == "production" else default_dev


# Unoconv
ISOLATE_UNOCONV = env.bool("ISOLATE_UNOCONV", default=False)
SECRET_KEY = env.str("SECRET_KEY", default=default("uuuuuuuuuu"))
DEBUG = env.bool("DEBUG", default=default(True, False))
ALLOWED_HOSTS = env.list("ALLOWED_HOSTS", default=default(["*"]))


# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases

DATABASE_DIR = env.str("DATABASE_DIR", default="/var/lib/document-merge-service/data")
DATABASES = {
    "default": {
        "ENGINE": env.str("DATABASE_ENGINE", default="django.db.backends.sqlite3"),
        "NAME": env.str("DATABASE_NAME", default=f"{DATABASE_DIR}/sqlite3.db"),
        "USER": env.str("DATABASE_USER", default=""),
        "PASSWORD": env.str("DATABASE_PASSWORD", default=""),
        "HOST": env.str("DATABASE_HOST", default=""),
        "PORT": env.str("DATABASE_PORT", default=""),
        "OPTIONS": env.dict("DATABASE_OPTIONS", default={}),
    }
}


# Application definition

INSTALLED_APPS = [
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "django_filters",
    "document_merge_service.api.apps.DefaultConfig",
    "corsheaders",
    "generic_permissions.apps.GenericPermissionsConfig",
]

if "postgresql" in DATABASES["default"]["ENGINE"]:  # pragma: no cover
    INSTALLED_APPS.append("django.contrib.postgres")

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.locale.LocaleMiddleware",
]

ROOT_URLCONF = "document_merge_service.urls"
WSGI_APPLICATION = "document_merge_service.wsgi.application"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]


# Cache
# https://docs.djangoproject.com/en/1.11/ref/settings/#caches

CACHES = {
    "default": {
        "BACKEND": env.str(
            "CACHE_BACKEND", default="django.core.cache.backends.locmem.LocMemCache"
        ),
        "LOCATION": env.str("CACHE_LOCATION", ""),
    }
}


# CORS
CORS_ORIGIN_ALLOW_ALL = env.bool("CORS_ORIGIN_ALLOW_ALL", False)
CORS_ORIGIN_REGEX_WHITELIST = [r"^(https?://)?127\.0\.0\.1:\d{4}$"]
CORS_ORIGIN_REGEX_WHITELIST += env.list(
    "CORS_ORIGIN_REGEX_WHITELIST", default=[r"^(https?://)?127\.0\.0\.1:\d{4}$"]
)


# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = env.str("LANGUAGE_CODE", "en-us")
TIME_ZONE = env.str("TIME_ZONE", "UTC")
USE_I18N = True
USE_TZ = True


def parse_admins(admins):
    """
    Parse env admins to django admins.

    Example of ADMINS environment variable:
    Test Example <test@example.com>,Test2 <test2@example.com>
    """
    result = []
    for admin in admins:
        match = re.search(r"(.+) \<(.+@.+)\>", admin)
        if not match:  # pragma: no cover
            raise environ.ImproperlyConfigured(
                'In ADMINS admin "{0}" is not in correct '
                '"Firstname Lastname <email@example.com>"'.format(admin)
            )
        result.append((match.group(1), match.group(2)))
    return result


ADMINS = parse_admins(env.list("ADMINS", default=[]))


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

STATIC_URL = "/static/"
STATIC_ROOT = env.str("STATIC_ROOT", None)

# Media files

STORAGES = {
    "default": {
        "BACKEND": env.str(
            "FILE_STORAGE", default="django.core.files.storage.FileSystemStorage"
        )
    },
    "staticfiles": {
        "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
    },
}
MEDIA_ROOT = env.str("MEDIA_ROOT", "")
# TODO: This should be removed in favor of storing the files in a bucket
# https://code.djangoproject.com/ticket/32991
MEDIA_URL = env.str("MEDIA_URL", "api/v1/template/")

# django-storages S3 settings
AWS_S3_ACCESS_KEY_ID = env.str("AWS_S3_ACCESS_KEY_ID", "")
AWS_S3_SECRET_ACCESS_KEY = env.str("AWS_S3_SECRET_ACCESS_KEY", "")
AWS_STORAGE_BUCKET_NAME = env.str("AWS_STORAGE_BUCKET_NAME", "")
AWS_S3_ENDPOINT_URL = env.str("AWS_S3_ENDPOINT_URL", "")
AWS_S3_REGION_NAME = env.str("AWS_S3_REGION_NAME", "")
AWS_LOCATION = env.str("AWS_LOCATION", "")
AWS_S3_FILE_OVERWRITE = env.bool("AWS_S3_FILE_OVERWRITE", False)
AWS_S3_SIGNATURE_VERSION = env.str("AWS_S3_SIGNATURE_VERSION", "v2")
AWS_S3_USE_SSL = env.bool("AWS_S3_USE_SSL", default=True)
AWS_S3_VERIFY = env.bool("AWS_S3_VERIFY", default=None)

# unoconv
UNOCONV_ALLOWED_TYPES = env.list("UNOCOV_ALLOWED_TYPES", default=["pdf"])
UNOCONV_PYTHON = env.str("UNOCONV_PYTHON", default="/usr/bin/python3")
UNOCONV_PATH = env.str("UNOCONV_PATH", default="/usr/bin/unoconv")


# Jinja2
DOCXTEMPLATE_JINJA_EXTENSIONS = env.list(
    "DOCXTEMPLATE_JINJA_EXTENSIONS", default=default([])
)

# Authentication

REQUIRE_AUTHENTICATION = env.bool("REQUIRE_AUTHENTICATION", False)

OIDC_USERINFO_ENDPOINT = env.str("OIDC_USERINFO_ENDPOINT", default=None)
OIDC_VERIFY_SSL = env.bool("OIDC_VERIFY_SSL", default=True)
OIDC_GROUPS_CLAIM = env.str("OIDC_GROUPS_CLAIM", default="")
OIDC_BEARER_TOKEN_REVALIDATION_TIME = env.int(
    "OIDC_BEARER_TOKEN_REVALIDATION_TIME", default=0
)

# Rest framework
# https://www.django-rest-framework.org/api-guide/settings/

REST_FRAMEWORK = {
    "DEFAULT_PERMISSION_CLASSES": [
        "document_merge_service.api.permissions.AsConfigured",
    ],
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "document_merge_service.api.authentication.BearerTokenAuthentication"
    ],
    "UNAUTHENTICATED_USER": "document_merge_service.api.authentication.AnonymousUser",
    "DEFAULT_FILTER_BACKENDS": (
        "rest_framework.filters.OrderingFilter",
        "django_filters.rest_framework.DjangoFilterBackend",
        "rest_framework.filters.SearchFilter",
    ),
    "TEST_REQUEST_DEFAULT_FORMAT": "json",
}

PAGINATION_ENABLED = env.bool("PAGINATION_ENABLED", True)
PAGINATION_DEFAULT_PAGE_SIZE = env.int("PAGINATION_DEFAULT_PAGE_SIZE", 100)
PAGINATION_MAX_PAGE_SIZE = env.int("PAGINATION_MAX_PAGE_SIZE", 1000)

if PAGINATION_ENABLED:
    REST_FRAMEWORK.update(
        {
            "DEFAULT_PAGINATION_CLASS": "document_merge_service.api.pagination.APIPagination",
            "PAGE_SIZE": PAGINATION_DEFAULT_PAGE_SIZE,
        }
    )

# Logging
ENABLE_ADMIN_EMAIL_LOGGING = env.bool("ENABLE_ADMIN_EMAIL_LOGGING", False)

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "level": "WARNING",
            "filters": None,
            "class": "logging.StreamHandler",
        },
        "mail_admins": {
            "level": "ERROR",
            "filters": None,
            "class": "django.utils.log.AdminEmailHandler",
        },
    },
    "loggers": {"django": {"handlers": ["console"], "level": "WARNING"}},
}

URL_PREFIX = env.str("URL_PREFIX", default="")

# Email settings
SERVER_EMAIL = env.str("SERVER_EMAIL", default="root@localhost")
DEFAULT_FROM_EMAIL = env.str("DEFAULT_FROM_EMAIL", default="webmaster@localhost")
EMAIL_HOST = env.str("EMAIL_HOST", default="localhost")
EMAIL_PORT = env.int("EMAIL_PORT", default=25)
EMAIL_HOST_USER = env.str("EMAIL_HOST_USER", default="")
EMAIL_HOST_PASSWORD = env.str("EMAIL_HOST_PASSWORD", default="")
EMAIL_USE_TLS = env.bool("EMAIL_USE_TLS", default=False)

# Email error handler
if ENABLE_ADMIN_EMAIL_LOGGING:  # pragma: no cover
    LOGGING["loggers"]["django"]["handlers"].append("mail_admins")  # type: ignore

# Sentry error tracking
SENTRY_DSN = env.str("SENTRY_DSN", default="")
SENTRY_ENVIRONMENT = env.str("SENTRY_ENVIRONMENT", default="development")
SENTRY_TRACES_SAMPLE_RATE = env.float("SENTRY_TRACES_SAMPLE_RATE", default=0.01)
SENTRY_SEND_DEFAULT_PII = env.bool("SENTRY_SEND_DEFAULT_PII", default=False)

if SENTRY_DSN:  # pragma: no cover
    sentry_init(
        SENTRY_DSN,
        SENTRY_ENVIRONMENT,
        SENTRY_TRACES_SAMPLE_RATE,
        SENTRY_SEND_DEFAULT_PII,
    )

# https://github.com/adfinis/django-generic-api-permissions
GENERIC_PERMISSIONS_PERMISSION_CLASSES = env.list("DMS_PERMISSION_CLASSES", default=[])
GENERIC_PERMISSIONS_VISIBILITY_CLASSES = env.list("DMS_VISIBILITY_CLASSES", default=[])

# App specific arguments for the extension classes
EXTENSIONS_ARGUMENTS = env.dict("EXTENSIONS_ARGUMENTS", default={})



import io

import openpyxl
import pytest
from rest_framework import exceptions

from ..data import django_file
from ..engines import XlsxTemplateEngine

_structure = {
    "key0": "xdata0",
    "key1": {
        "subkey1": "xdata1",
    },
    "key2": [
        "mixed",
        "list",
        {"subkey2": "xdata2"},
    ],
}

_expect_name = ["test", "another"]


def test_render():
    tmpl = django_file("xlsx-structure.xlsx")
    engine = XlsxTemplateEngine(tmpl)
    buf = io.BytesIO()
    engine.merge(_structure, buf)
    buf.seek(0)
    doc = openpyxl.load_workbook(buf)
    for i, ws in enumerate(doc.worksheets):
        assert ws.title == _expect_name[i]
        assert ws["A1"].value == "xdata0"
        assert ws["A2"].value == "xdata1"
        assert ws["A3"].value == _expect_name[i]
        assert ws["A5"].value == "Item: mixed"
        assert ws["A6"].value == "Item: list"
        assert ws["A7"].value == "Subitem: xdata2"


@pytest.mark.parametrize(
    "available, expect_success",
    [
        ([], True),  # disabled check by empty list
        (["key0"], False),
        (["key1.subkey1"], False),
        (["key0", "key1"], False),
        (["key0", "key1.subkey1", "key2[].subkey2"], True),  # full set of vars
    ],
)
def test_validate_template(available, expect_success):
    tmpl = django_file("xlsx-structure.xlsx")
    engine = XlsxTemplateEngine(tmpl)
    if expect_success:
        engine.validate(available)
    else:
        with pytest.raises(exceptions.ValidationError):
            engine.validate(available)


def test_syntax_error():
    tmpl = django_file("xlsx-syntax.xlsx")
    engine = XlsxTemplateEngine(tmpl)
    with pytest.raises(exceptions.ValidationError):
        engine.validate(sample_data=_structure)


def test_valid_error():
    tmpl = django_file("xlsx-not-valid.xlsx")
    engine = XlsxTemplateEngine(tmpl)
    with pytest.raises(exceptions.ParseError):
        engine.validate(sample_data=_structure)



from pathlib import Path
from tempfile import NamedTemporaryFile

from django.conf import settings
from django.http import HttpResponse

from .unoconv import Unoconv


class FileConverter:
    def convert(file_contents, target_format):
        dir = Path(settings.DATABASE_DIR, "tmp")
        dir.mkdir(parents=True, exist_ok=True)

        with NamedTemporaryFile("wb", dir=dir) as tmp:
            tmp.write(file_contents)
            unoconv = Unoconv(
                pythonpath=settings.UNOCONV_PYTHON,
                unoconvpath=settings.UNOCONV_PATH,
            )
            result = unoconv.process(tmp.name, target_format)

        status = 200 if result.returncode == 0 else 500

        return HttpResponse(
            content=result.stdout, status=status, content_type=result.content_type
        )



import pytest
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from ..api.unoconv import Unoconv


def test_get_unoconv_formats():
    uno = Unoconv(pythonpath=settings.UNOCONV_PYTHON, unoconvpath=settings.UNOCONV_PATH)
    formats = uno.get_formats()
    assert "pdf" in formats


def test_get_unoconv_formats_invalid_format(monkeypatch):
    monkeypatch.setattr(settings, "UNOCONV_ALLOWED_TYPES", ["invalid"])
    uno = Unoconv(pythonpath=settings.UNOCONV_PYTHON, unoconvpath=settings.UNOCONV_PATH)

    with pytest.raises(ImproperlyConfigured):
        uno.get_formats()



import functools
import hashlib

import requests
from django.conf import settings
from django.core.cache import cache
from django.utils.encoding import force_bytes, smart_str
from django.utils.translation import gettext as _
from rest_framework import authentication, exceptions


class AnonymousUser(object):
    def __init__(self):
        self.username = None
        self.groups = []

    @property
    def group(self):
        return self.groups and self.groups[0]

    @property
    def is_authenticated(self):
        return False

    def __str__(self):
        return "AnonymousUser"


class AuthenticatedUser(AnonymousUser):
    def __init__(self, userinfo):
        self.username = userinfo["sub"]
        groups = []
        if settings.OIDC_GROUPS_CLAIM:
            groups = userinfo[settings.OIDC_GROUPS_CLAIM]
        self.groups = groups
        self.userinfo = userinfo

    @property
    def is_authenticated(self):
        return True

    def __str__(self):
        return self.username


class BearerTokenAuthentication(authentication.BaseAuthentication):
    header_prefix = "Bearer"

    def get_bearer_token(self, request):
        auth = authentication.get_authorization_header(request).split()

        if not auth:
            return None

        if smart_str(auth[0].lower()) != self.header_prefix.lower():
            raise exceptions.AuthenticationFailed(_("No Bearer Authorization header"))

        if len(auth) == 1:
            msg = _("Invalid Authorization header. No credentials provided")
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = _(
                "Invalid Authorization header. Credentials string should "
                "not contain spaces."
            )
            raise exceptions.AuthenticationFailed(msg)

        return auth[1]

    def get_userinfo(self, token):
        response = requests.get(
            settings.OIDC_USERINFO_ENDPOINT,
            verify=settings.OIDC_VERIFY_SSL,
            headers={"Authorization": f"Bearer {smart_str(token)}"},
        )

        try:
            response.raise_for_status()
        except requests.HTTPError as e:
            raise exceptions.AuthenticationFailed(
                f"Retrieving userinfo from {settings.OIDC_USERINFO_ENDPOINT} "
                f"failed with error '{str(e)}'."
            )

        return response.json()

    def authenticate(self, request):
        if not settings.REQUIRE_AUTHENTICATION:
            return None

        token = self.get_bearer_token(request)
        if token is None:
            return None

        userinfo_method = functools.partial(self.get_userinfo, token=token)
        # token might be too long for key so we use hash sum instead.
        hashsum_token = hashlib.sha256(force_bytes(token)).hexdigest()
        userinfo = cache.get_or_set(
            f"authentication.userinfo.{hashsum_token}",
            userinfo_method,
            timeout=settings.OIDC_BEARER_TOKEN_REVALIDATION_TIME,
        )

        return (
            AuthenticatedUser(userinfo),
            token,
        )

    def authenticate_header(self, request):
        return f"{self.header_prefix} realm={settings.OIDC_USERINFO_ENDPOINT}"



import pytest
from docx.shared import Mm

from document_merge_service.api.data import django_file

from ..jinja import (
    dateformat,
    datetimeformat,
    emptystring,
    getwithdefault,
    image,
    timeformat,
)


@pytest.mark.parametrize(
    "inp,locale,expected", [("2019-12-31", "de_CH", "31.12.2019"), (None, "de_CH", "")]
)
def test_dateformat(inp, expected, locale):
    formatted = dateformat(inp, locale=locale)
    assert formatted == expected


@pytest.mark.parametrize(
    "inp,locale,expected",
    [("2019-12-31 23:59", "de_CH", "31.12.2019, 23:59:00"), (None, "de_CH", "")],
)
def test_datetimeformat(inp, expected, locale):
    formatted = datetimeformat(inp, locale=locale)
    assert formatted == expected


@pytest.mark.parametrize(
    "inp,locale,expected", [("23:59", "de_CH", "23:59:00"), (None, "de_CH", "")]
)
def test_timeformat(inp, expected, locale):
    formatted = timeformat(inp, locale=locale)
    assert formatted == expected


@pytest.mark.parametrize("inp,expected", [("text", "text"), (None, "")])
def test_emptystring(inp, expected):
    formatted = emptystring(inp)
    assert formatted == expected


@pytest.mark.parametrize(
    "inp,default,expected",
    [("text", "", "text"), (None, "", ""), (None, "something", "something")],
)
def test_getwithdefault(inp, default, expected):
    formatted = getwithdefault(inp, default=default)
    assert formatted == expected


@pytest.mark.parametrize(
    "width,height,keep_aspect_ratio,expected_size",
    [
        (20, 10, False, (20, 10)),
        (20, 10, True, (10, 10)),
        (10, 20, True, (10, 10)),
        (10, None, False, (10, None)),
        (None, 10, False, (None, 10)),
        (10, None, True, (10, None)),
        (None, 10, True, (None, 10)),
    ],
)
def test_image(width, height, keep_aspect_ratio, expected_size):
    # The size of "black.png" is 32x32 pixels which is an aspect ratio of 1
    inline_image = image(
        {"_tpl": None, "black.png": django_file("black.png").file},
        "black.png",
        width,
        height,
        keep_aspect_ratio,
    )

    w, h = expected_size

    assert inline_image.width == (Mm(w) if w else None)
    assert inline_image.height == (Mm(h) if h else None)



import os.path
import shutil

from django.conf import settings
from django.core.files import File

_data_path = os.path.dirname(os.path.realpath(__file__))


def django_file(name, mode="rb", new_path=None, new_name=None):
    abspath = os.path.join(_data_path, name)

    if not new_path:
        new_path = settings.MEDIA_ROOT

    if not new_name:
        new_name = name

    try:
        os.makedirs(new_path)
    except FileExistsError:  # pragma: no cover
        pass

    shutil.copy(abspath, f"{new_path}/{new_name}")

    return File(open(abspath, mode), name=new_name)



import hashlib
import json

import pytest
from django.core.cache import cache
from rest_framework import exceptions, status

from .. import authentication


@pytest.mark.parametrize(
    "authentication_header,status_code,error",
    [
        ("", status.HTTP_200_OK, False),
        ("Bearer", status.HTTP_200_OK, True),
        ("Bearer Too many params", status.HTTP_200_OK, True),
        ("Basic Auth", status.HTTP_200_OK, True),
        ("Bearer Token", status.HTTP_200_OK, False),
        ("Bearer Token", status.HTTP_502_BAD_GATEWAY, True),
    ],
)
def test_bearer_token_authentication_authenticate(
    rf, authentication_header, error, requests_mock, settings, status_code
):
    settings.REQUIRE_AUTHENTICATION = True

    userinfo = {"sub": "1"}
    requests_mock.get(
        settings.OIDC_USERINFO_ENDPOINT,
        status_code=status_code,
        request_headers={"Authorization": authentication_header},
        text=json.dumps(userinfo),
    )

    request = rf.get("/openid", HTTP_AUTHORIZATION=authentication_header)

    try:
        result = authentication.BearerTokenAuthentication().authenticate(request)
    except exceptions.AuthenticationFailed:
        assert error
    else:
        if result:
            user, auth = result
            assert user.is_authenticated
            assert (
                cache.get(
                    f"authentication.userinfo.{hashlib.sha256(b'Token').hexdigest()}"
                )
                == userinfo
            )


def test_bearer_token_authentication_authenticate_groups_claim(
    settings, requests_mock, rf
):
    settings.REQUIRE_AUTHENTICATION = True
    settings.OIDC_GROUPS_CLAIM = "document-merge-service"

    userinfo = {"sub": "1", settings.OIDC_GROUPS_CLAIM: ["test"]}
    requests_mock.get(settings.OIDC_USERINFO_ENDPOINT, text=json.dumps(userinfo))

    request = rf.get("/openid", HTTP_AUTHORIZATION="Bearer Token")
    user, auth = authentication.BearerTokenAuthentication().authenticate(request)
    assert user.is_authenticated
    assert user.group == "test"
    assert user.groups == ["test"]


def test_bearer_token_authentication_header(rf):
    request = rf.get("/openid")
    assert (
        authentication.BearerTokenAuthentication().authenticate_header(request)
        == "Bearer realm=mock://document-merge-service.github.com/openid/userinfo"
    )



from django.conf import settings
from rest_framework import permissions


class AsConfigured(permissions.IsAuthenticated):
    def has_permission(self, request, view):
        if settings.REQUIRE_AUTHENTICATION:
            return super().has_permission(request, view)
        return True



import logging
import os
import re
import signal
from collections import namedtuple
from mimetypes import guess_type
from subprocess import (
    PIPE,
    CalledProcessError,
    CompletedProcess,
    Popen,
    TimeoutExpired,
    run as srun,
)
from uuid import uuid4

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

logger = logging.getLogger(__name__)
_unshare = "unshare"

UnoconvResult = namedtuple(
    "UnoconvResult", ["stdout", "stderr", "returncode", "content_type"]
)

# in testing 2 seconds is enough
_min_timeout = 2

# terminate_then_kill() takes 1 second in the worst case, so we have to use three seconds,
# or the timeout won't be triggered before harakiri.
# Increased to 3 seconds to be safe, we have reports of orphan soffice.bin in production.
_ahead_of_harakiri = 3


def get_default_timeout():
    timeout = 55
    try:  # pragma: no cover
        import uwsgi

        harakiri = uwsgi.opt.get("harakiri")
        if harakiri:
            try:
                timeout = max(int(harakiri) - _ahead_of_harakiri, _min_timeout)
            except ValueError:
                pass
    except ModuleNotFoundError:
        pass
    return timeout


_default_timeout = get_default_timeout()


def getpgid(proc):
    try:
        return (proc, os.getpgid(proc.pid))
    except ProcessLookupError:  # pragma: no cover
        return (proc, None)


def kill(proc, sig):
    process, group = proc
    try:
        if group is None:
            if process.returncode is None:  # pragma: no cover
                os.kill(process.pid, sig)
        else:
            os.killpg(group, sig)
    except ProcessLookupError:
        pass


def terminate_then_kill(proc):
    process, _ = proc
    kill(proc, signal.SIGTERM)
    try:
        process.wait(timeout=1)
    except TimeoutExpired:  # pragma: no cover
        pass
    finally:
        kill(proc, signal.SIGKILL)


def run_fork_safe(
    *popenargs,
    input=None,
    capture_output=False,
    timeout=None,
    check=False,
    **kwargs,
):
    """Run command with arguments and return a CompletedProcess instance.

    Works like `subprocess.run`, but puts the subprocess and its children in a new
    process group, so orphan forks can be terminated, too.
    """
    if input is not None:  # pragma: no cover
        if kwargs.get("stdin") is not None:
            raise ValueError("stdin and input arguments may not both be used.")
        kwargs["stdin"] = PIPE

    if capture_output:  # pragma: no cover
        if kwargs.get("stdout") is not None or kwargs.get("stderr") is not None:
            raise ValueError(
                "stdout and stderr arguments may not be used " "with capture_output."
            )
        kwargs["stdout"] = PIPE
        kwargs["stderr"] = PIPE

    with Popen(*popenargs, start_new_session=True, **kwargs) as process:
        proc = getpgid(process)
        try:
            stdout, stderr = process.communicate(input, timeout=timeout)
        finally:
            terminate_then_kill(proc)
        retcode = process.poll()
        if check and retcode:  # pragma: no cover
            raise CalledProcessError(
                retcode, process.args, output=stdout, stderr=stderr
            )
    return CompletedProcess(process.args, retcode, stdout, stderr)


def run(cmd, unshare=False):
    # Run libreoffice in isolation. If the main broker of libreoffice locks up,
    # all following calls to unoconv will hang as well. By unsharing the mount namespace,
    # we get a copy of all the mounts, but we can change them without affecting the
    # original. So unoconv will never connect to a main broker, and allways start a
    # new instance. Then run_fork_safe will terminate the process and all the children
    # of unoconv.
    #
    # I think masking /tmp is enough, but we can use this technique for other paths if
    # needed.
    if unshare:
        shell = [
            "unshare",
            "--map-root-user",
            "--ipc",
            "--mount",
            "sh",
        ]
        cmd = f"""
            mount -t tmpfs tmpfs /tmp
            exec {cmd}
        """.strip()
    else:  # pragma: no cover
        shell = ["sh"]
    ret = run_fork_safe(
        shell,
        stdout=PIPE,
        stderr=PIPE,
        timeout=_default_timeout,
        input=cmd.encode("utf-8"),
    )
    return ret


class Unoconv:
    def __init__(self, pythonpath, unoconvpath):
        """
        Convert documents with unoconv command-line utility.

        :param pythonpath: str() - path to the python interpreter
        :param unoconvpath: str() - path to the unoconv binary
        """
        self.cmd = f"{pythonpath} {unoconvpath}"
        self.unshare = settings.ISOLATE_UNOCONV

    def get_formats(self):
        from django.conf import settings

        cmd = f"{self.cmd} --show"
        p = run(cmd, unshare=self.unshare)
        if not p.returncode == 0:  # pragma: no cover
            raise Exception("Failed to fetch the formats from unoconv!")

        formats = []
        for line in p.stderr.decode("utf-8").split("\n"):
            if line.startswith("  "):
                match = re.match(r"^\s\s(?P<format>[a-z]*)\s", line)
                if match:
                    formats.append(match.group("format"))

        formats = set(formats)
        not_supported = set(settings.UNOCONV_ALLOWED_TYPES) - formats
        if not_supported:
            raise ImproperlyConfigured(
                f"Unoconv doesn't support types {', '.join(not_supported)}."
            )

        return formats

    def process(self, filename, convert):
        """
        Convert a file.

        :param filename: str()
        :param convert: str() - target format. e.g. "pdf"
        :return: UnoconvResult()
        """
        # unoconv MUST be running with the same python version as libreoffice
        pipe = str(uuid4())
        cmd = f"{self.cmd} --timeout 10 --pipe {pipe} --format {convert} --stdout '{filename}'"
        p = run(cmd, unshare=self.unshare)
        stdout = p.stdout
        if not p.returncode == 0:  # pragma: no cover
            stdout = f"unoconv returncode: {p.returncode}"
            try:
                srun([_unshare, "true"], check=True)
            except CalledProcessError:
                logger.error(
                    "Could not unshare, this process needs CAP_SYS_ADMIN to unshare."
                )
            else:
                logger.error(
                    f"unoconv failed with returncode: {p.returncode} stderr: {p.stderr}"
                )
                if p.stdout:
                    logger.error(f"unoconv failed with stdout: {p.stdout}")

        content_type, _ = guess_type(f"something.{convert}")

        result = UnoconvResult(
            stdout=stdout,
            stderr=p.stderr,
            returncode=p.returncode,
            content_type=content_type,
        )

        return result
